#include <SPI.h>
#include <SD.h>
#include <SoftwareSerial.h>

#define THRESHOLD 0x40

SoftwareSerial mySerial(5, 6); // RX, TX
int kbd_abcde_nopqr = A0, A0_value;
int kbd_fghi_stuv = A1, A1_value;
int kbd_jklm_wxyz = A2, A2_value;
int kbd_space_shift_enter = A3, A3_value;
int second_half = 0;
// ranges = {{1000, 1050}, {950, 1000}, {850, 900}, {450, 550}, {50, 150}}
int kbd_button_index[] = {4, 4, -1, -1, 3, 3, -1, -1, 2, 1, 0};
char kbd_buttons[2][4][5] = { { {'A', 'B', 'C', 'D', 'E'},
                                {'F', 'G', 'H', 'I'},
                                {'J', 'K', 'L', 'M'},
                                {' ', '0', '\n'}  },
                                { {'N', 'O', 'P', 'Q', 'R'},
                                  {'S', 'T', 'U', 'V'},
                                  {'W', 'X', 'Y', 'Z'},
                                  {'/', '0', '\n'}  } };
char cmd[50];
char output[150];
int cmd_index, output_index;

void list(File dir, int level) {  
  int len;
  while (true) {
    File entry =  dir.openNextFile();

    if (!entry) {
      return;
    }

    for (uint8_t i = 0; i < level; i++) {
      output[output_index++] = '-';
      output[output_index++] = '-';
    }

    len = strlen(entry.name());
    memcpy(output + output_index, entry.name(), len);
    output_index += len;

    output[output_index++] = '<';
    output[output_index++] = 'b';
    output[output_index++] = 'r';
    output[output_index++] = '>';
 
    if (entry.isDirectory()) {
      output[output_index++] = '<';
      output[output_index++] = 'b';
      output[output_index++] = 'r';
      output[output_index++] = '>';

      list(entry, level + 1);

    }

    entry.close();
  }
}

void cmd_exec() {
  char *p = strtok(cmd, " ");
  File file, dir;
  int offset, bytes_count;

  if (p) {
    if (*p == 'L') {
      p = strtok(NULL, " ");

      dir = SD.open(p);

      list(dir, 0);

      output[output_index] = 0;
      pr_dbg();
      mySerial.write(output, sizeof(output));
      pr_dbg();
      output_index = 0;

      return;

    } else if (*p == 'R') { // read
      p = strtok(NULL, " ");
      if (!SD.exists(p)) {
        memcpy(output + output_index, "No such file or directory!", sizeof("No such file or directory!"));
        output_index += sizeof("No such file or directory!");

        output[output_index] = 0;
        pr_dbg();
        mySerial.write(output, sizeof(output));
        pr_dbg();
        output_index = 0;
        
        return;
      }

      Serial.println(p);
      file = SD.open(p, FILE_READ);

      char c = 0;
      c = file.read();
      while (c != -1) {
        output[output_index++] = c;
        c = file.read();
      }

      file.close();

      output[output_index] = 0;
      pr_dbg();
      mySerial.write(output, sizeof(output));
      pr_dbg();
      output_index = 0;

      return;

    } else if (*p == 'W') { // write
      p = strtok(NULL, " ");

      Serial.println(p);
      file = SD.open(p, FILE_WRITE);

      p = strtok(NULL, " ");

      Serial.println(p);
      file.write(p, strlen(p));

      file.close();

      return;

    } else {
      Serial.println("lool");
      memcpy(output + output_index, "No such command!\n", sizeof("No such command!<br>"));
      output_index += sizeof("No such command!<br>");

      output[output_index] = 0;
      pr_dbg();
      mySerial.write(output, sizeof(output));
      pr_dbg();
      output_index = 0;

      return;
    }
  }
}

void setup() {
  mySerial.begin(4800);

  if (!SD.begin(10)) {
    while (1);
  }
}

void loop() {
  char c;

  A0_value = analogRead(A0);
  if (A0_value > THRESHOLD) {
    c = kbd_buttons[second_half][0][kbd_button_index[A0_value / 100]];

    cmd[cmd_index++] = c;

    output[output_index++] = c;
    output[output_index] = 0;
    pr_dbg();
    mySerial.write(output, sizeof(output));
    pr_dbg();
  }

  A1_value = analogRead(A1);
  if (A1_value > THRESHOLD) {
    c = kbd_buttons[second_half][1][kbd_button_index[A1_value / 100]];

    cmd[cmd_index++] = c;

    output[output_index++] = c;
    output[output_index] = 0;
    pr_dbg();
    mySerial.write(output, sizeof(output));
    pr_dbg();
  }

  A2_value = analogRead(A2);
  if (A2_value > THRESHOLD) {
    c = kbd_buttons[second_half][2][kbd_button_index[A2_value / 100]];

    cmd[cmd_index++] = c;
    
    output[output_index++] = c;
    output[output_index] = 0;
    pr_dbg();
    mySerial.write(output, sizeof(output));
    pr_dbg();
  }

  A3_value = analogRead(A3);
  if (A3_value > THRESHOLD) {
    c = kbd_buttons[second_half][3][kbd_button_index[A3_value / 100]];

    switch (c) {
    case '/':
    case ' ':
      cmd[cmd_index++] = c;

      output[output_index++] = c;
      output[output_index] = 0;
      pr_dbg();
      mySerial.write(output, sizeof(output));
      pr_dbg();

      break;

    case '\n':
      output[output_index++] = '<';
      output[output_index++] = 'b';
      output[output_index++] = 'r';
      output[output_index++] = '>';
      output[output_index] = 0;

      cmd_exec();
      memset(cmd, 0, sizeof(cmd));
      cmd_index = 0;
      output_index = 0;
      memset(output, 0, sizeof(output));

      break;

    case '0':
      second_half ^= 1;

      analogWrite(9, second_half);

      break;
    }
  }

  delay(150);
}
